package business;

import java.awt.*;
import javax.swing.*;
import javax.sound.midi.*;
import javax.sound.sampled.AudioSystem;
import java.util.Vector;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.BufferedInputStream;

/**
 * Illustrates general MIDI melody instruments and MIDI controllers.
 *
 * @version @(#)MidiSynth.java	1.15 99/12/03
 * @author Brian Lichtenwalter
 */
public class MidiSynth implements ControlContext {

    final int PROGRAM = 192;
    final int NOTEON = 144;
    final int NOTEOFF = 128;
    long currentTime=0;
    Sequencer sequencer;
    Sequence sequence;
    Synthesizer synthesizer;
    Instrument instruments[];
    ChannelData channels[];
    ChannelData cc;    // current channel
    boolean record;
    Track track;
    long startTime;
    RecordFrame recordFrame;
    int DELAY=100;




    public MidiSynth() {
    }


    public void open() {
        try {
            if (synthesizer == null) {
                if ((synthesizer = MidiSystem.getSynthesizer()) == null) {
                    System.out.println("getSynthesizer() failed!");
                    return;
                }
            }
            synthesizer.open();
            sequencer = MidiSystem.getSequencer();
            sequence = new Sequence(Sequence.PPQ, 10);
        } catch (Exception ex) { ex.printStackTrace(); return; }

        Soundbank sb = synthesizer.getDefaultSoundbank();
        if (sb != null) {
            instruments = synthesizer.getDefaultSoundbank().getInstruments();
            synthesizer.loadInstrument(instruments[124]);
        }
        MidiChannel midiChannels[] = synthesizer.getChannels();
        channels = new ChannelData[midiChannels.length];
        for (int i = 0; i < channels.length; i++) {
            channels[i] = new ChannelData(midiChannels[i], i);
        }
        cc = channels[0];

    }


    public void close() {
        if (synthesizer != null) {
            synthesizer.close();
        }
        if (sequencer != null) {
            sequencer.close();
        }
        sequencer = null;
        synthesizer = null;
        instruments = null;
        channels = null;
        if (recordFrame != null) {
            //recordFrame.dispose();
            recordFrame = null;
        }
    }





    /**
     * Stores MidiChannel information.
     */

    class ChannelData {

        MidiChannel channel;
        boolean solo, mono, mute, sustain;
        int velocity, pressure, bend, reverb;
        // int row, col;
        int num;

        public ChannelData(MidiChannel channel, int num) {
            this.channel = channel;
            this.num = num;
            velocity = pressure = reverb = 64;
            bend = 8192;
        }
    } // End class ChannelData

    public void programChange(int program) {
        if (instruments != null) {
            synthesizer.loadInstrument(instruments[program]);
        }
        cc.channel.programChange(program);
        //if (record) {
        createShortEvent(PROGRAM, program);
        //}
    }

    //createShortEvent(PROGRAM,cc.col*8+cc.row);
    public void createShortEvent(int type, int num) {

        //System.out.println("Coming inside CreateShortEvent");
        ShortMessage message = new ShortMessage();
        try {
            long millis;
            /*if(type==NOTEOFF)
            millis = currentTime - startTime;
            else*/
            currentTime=System.currentTimeMillis();
            millis = currentTime - startTime;

            long tick = millis * sequence.getResolution() / 500;
            message.setMessage(type+cc.num, num, cc.velocity);
            MidiEvent event = new MidiEvent(message, tick);
            track.add(event);

            /* try
            {
            Thread.sleep(500);
            }
            catch(Exception e)
            {

            }
*/
        } catch (Exception ex) { ex.printStackTrace(); }
        //System.out.println("Exiting inside CreateShortEvent");
    }

    public void createShortEventOFF(int type, int num) {

        //System.out.println("Coming inside CreateShortEvent");
        ShortMessage message = new ShortMessage();
        try {
            long millis;
            /*if(type==NOTEOFF)
            millis = currentTime - startTime;
            else*/
            currentTime+=DELAY;
            millis = currentTime - startTime;
            long tick = millis * sequence.getResolution() / 200;
            message.setMessage(type+cc.num, num, cc.velocity);
            MidiEvent event = new MidiEvent(message, tick);
            track.add(event);

            /*  try
            {
            Thread.sleep(500);
            }
            catch(Exception e)
            {

            }*/

        } catch (Exception ex) { ex.printStackTrace(); }
        //System.out.println("Exiting inside CreateShortEvent");
    }




    /**
     * A frame that allows for midi capture & saving the captured data.
     */
    class RecordFrame {//implements MetaEventListener {

        Vector tracks = new Vector();
        public RecordFrame() {
            //   sequencer.addMetaEventListener(this);
            try {
                sequence = new Sequence(Sequence.PPQ, 10);
            } catch (Exception ex) { ex.printStackTrace(); }
        }



        public void record(int num)
        {
            // track = sequence.createTrack();
            //startTime = System.currentTimeMillis();

            // add a program change right at the beginning of
            // the track for the current instrument
            createShortEvent(PROGRAM, num);
            //createShortEvent(PROGRAM,cc.col*8+cc.row);

        }

        public void saveMIDI()
        {
            try {
                File file = new File(System.getProperty("user.dir"));
                JFileChooser fc = new JFileChooser(file);
                fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
                    public boolean accept(File f) {
                        if (f.isDirectory()) {
                            return true;
                        }
                        return false;
                    }
                    public String getDescription() {
                        return "Save as .mid file.";
                    }
                });
                if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                    saveMidiFile(fc.getSelectedFile());
                }
            } catch (SecurityException ex) {
                //JavaSound.showInfoDialog();
                ex.printStackTrace();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }

        /*       public void meta(MetaMessage message) {
        if (message.getType() == 47) {  // 47 is end of track
        playB.setText("Play");
        recordB.setEnabled(true);
        }
        }*/
        public void play()
        {
            try {
                sequencer.open();
                sequencer.setSequence(sequence);
            } catch (Exception ex) { ex.printStackTrace(); }
            sequencer.start();
            //sequencer.get

        }



        class TrackData extends Object {
            Integer chanNum; String name; Track track;
            public TrackData(int chanNum, String name, Track track) {
                this.chanNum = new Integer(chanNum);
                this.name = name;
                this.track = track;
            }
        } // End class TrackData
    } // End class RecordFrame

    public void on(int kNum, boolean record) {
        //setNoteState(ON);
        //cc.channel.noteOn(kNum, cc.velocity);
        if (record) {
            System.out.println("ON______________REC");
            createShortEvent(NOTEON, kNum);
        }
    }
    public void off(int kNum, boolean record) {
        //setNoteState(OFF);
        //cc.channel.noteOff(kNum, cc.velocity);
        if (record) {
            System.out.println("OFF#######______________REC");
            createShortEvent(NOTEOFF, kNum);
        }
    }
    public void saveMidiFile(File file) {
        try {
            System.out.println("STARTED SAVING...................");
            int[] fileTypes = MidiSystem.getMidiFileTypes(sequence);
            if (fileTypes.length == 0) {
                System.out.println("Can't save sequence");
            } else {
                if (MidiSystem.write(sequence, fileTypes[0], file) == -1) {
                    throw new IOException("Problems writing to file");
                }
            }
            System.out.println("END OF SAVE PROCESS");
        } catch (SecurityException ex) {
            //JavaSound.showInfoDialog();
            System.out.println("FAILED TO SAVE");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void startTimer()
    {
        track = sequence.createTrack();
        startTime = System.currentTimeMillis();
        currentTime=startTime;
    }
    public Sequencer retrieveRecordedProgram()
    {
        File fProgram = new File("c:/code/mi.mid");
        String sCurrentName =  fProgram.getName();
        Object currentSound = null;

        try {
            currentSound = AudioSystem.getAudioInputStream(fProgram);
        } catch(Exception e1) {
            // load midi & rmf as inputstreams for now
            try {
                FileInputStream is = new FileInputStream(fProgram);
                currentSound = new BufferedInputStream(is, 1024);
            } catch (Exception e3) {
                //e3.printStackTrace();
                currentSound = null;
                return null;
            }
        }
        /* // user pressed stop or changed tabs while loading
        if (sequencer == null) {
        currentSound = null;
        return null;
        }*/
        if (currentSound instanceof Sequence || currentSound instanceof BufferedInputStream) {
            try {
                sequencer.open();
                if (currentSound instanceof Sequence) {
                    sequencer.setSequence((Sequence) currentSound);
                } else {
                    sequencer.setSequence((BufferedInputStream) currentSound);
                }
            } catch (InvalidMidiDataException imde) {
                System.out.println("Unsupported audio file.");
                currentSound = null;
                return null;
            } catch (Exception ex) {
                //ex.printStackTrace();
                currentSound = null;
                return null;
            }
        }
        Sequence seq=sequencer.getSequence();
        if(seq==null) System.out.println("SEQ==NULL");
        Track t[]=seq.getTracks();

        RecordFrame rf= new RecordFrame();
        startTimer();

      for(int i=0;i<t.length;i++)
      {
          for(int j=0;j<t[i].size();j++)
          {
              if(t[i].get(j).getMessage() instanceof MetaMessage){
              //System.out.println("event;  status="+t[i].get(j).getMessage().getStatus());
              //System.out.println(": byte=="+t[i].get(j).getMessage().toString());
              //System.out.println(": len=="+t[i].get(j).getMessage().getLength());
              }else{
                  int data1=((ShortMessage)t[i].get(j).getMessage()).getData1();
                          int cmd=((ShortMessage)t[i].get(j).getMessage()).getCommand();
                System.out.println("EVENt: data1= "+((ShortMessage)t[i].get(j).getMessage()).getData1()+": data2="+((ShortMessage)t[i].get(j).getMessage()).getData2());
                System.out.println("   ;;; cmd="+((ShortMessage)t[i].get(j).getMessage()).getCommand());
                if(cmd==192 || cmd==128 || cmd==144)
                {
                    createShortEvent(cmd, data1);
                    if(cmd==144)
                    {
                    try{
                        Thread.sleep(200);
                    }catch(Exception e)
                    {
                        System.out.println("SLEPT...............");
                    }
                    }
                }

              }
          }
      }
      rf.play();
        return sequencer;
    }

    public void startRec()
    {
        recordFrame=new RecordFrame();
    }
    public void rec()
    {
      retrieveRecordedProgram();
    }
    public void loopRec()
    {
        if(recordFrame == null) System.out.println("I AM NULL");
        recordFrame.record(40);
        createShortEvent(NOTEON,40);
        createShortEvent(NOTEOFF,40);
        try
        {
            Thread.sleep(400);
        }catch(Exception e)
        {

        }
    }

    public  void singleRec()
    {
        if(recordFrame == null) System.out.println("I AM NULL SINGLE-------");
        recordFrame.record(1);
        createShortEvent(NOTEON,20);
        createShortEvent(NOTEOFF,20);
        try
        {
            Thread.sleep(100);
        }catch(Exception e)
        {

        }

    }

    public void test()
    {
        RecordFrame rf=new RecordFrame();
        try{
            int inst=94;
            startTimer();
            rf.record(inst);
            //cc.num=9;
            synthesizer.loadInstrument(instruments[inst]);
            cc.channel.programChange(inst);

            /*createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,70);

            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);

            createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,70);

            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);

            createShortEvent(NOTEON,62);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,62);

            createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,70);


            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);
*/
            createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            /*createShortEvent(NOTEOFF,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);
            Thread.sleep(100);
*/

            createShortEvent(NOTEON,62);
            Thread.sleep(100);
           /* createShortEvent(NOTEOFF,62);

            createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);
            Thread.sleep(100);
            createShortEvent(NOTEON,61);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,61);
            createShortEvent(NOTEON,70);
            Thread.sleep(100);
            createShortEvent(NOTEON,67);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,70);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,67);
            Thread.sleep(100);
            createShortEvent(NOTEON,60);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,60);
            Thread.sleep(100);
            createShortEvent(NOTEON,58);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,58);
            Thread.sleep(100);
            createShortEvent(NOTEON,60);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,60);
            Thread.sleep(100);
            createShortEvent(NOTEON,58);
            Thread.sleep(100);
            createShortEvent(NOTEOFF,58);
            Thread.sleep(100);
*/

            //    createShortEventOFF(NOTEOFF,25);


            /*createShortEvent(NOTEON,31);
            createShortEventOFF(NOTEOFF,31);

            createShortEvent(NOTEON,42);
            createShortEventOFF(NOTEOFF,42);

            createShortEvent(NOTEON,52);
            createShortEventOFF(NOTEOFF,52); */


            /* cc.channel.noteOn(95, cc.velocity);

            cc.channel.noteOff(95, cc.velocity);
            Thread.sleep(250);


            cc.channel.noteOn(90, cc.velocity);

            cc.channel.noteOff(90, cc.velocity);

            Thread.sleep(250);
            //  Thread.sleep(1000);
            //createShortEvent(NOTEON,0);
            //createShortEvent(NOTEOFF,0);
            Thread.sleep(250);*/

            /*          rf.startTimer();
            rf.record(40);
            //programChange(40);

            // createShortEvent(NOTEON,20);
            on(25,true);
            //createShortEvent(NOTEON,25);
            off(25,true);
rf.record(50);
on(35,true);
//createShortEvent(NOTEON,35);
off(35,true);
rf.record(2);
on(45,true);
//createShortEvent(NOTEON,45);
off(45,true);
rf.record(10);
on(55,true);
//createShortEvent(NOTEON,55);
off(55,true);
rf.record(66);
on(60,true);
//createShortEvent(NOTEON,60);
off(60,true);
rf.record(70);
on(80,true);
//createShortEvent(NOTEON,80);
off(80,true);


//rf.record(10);
//programChange(68);
/*createShortEvent(NOTEON,40);
programChange(48);
createShortEvent(NOTEON,60);*/

            // System.out.println("1/2");

            // System.out.println("REC---FIN");
            // rf.saveMIDI();
            // saveMidiFile(new File("test.mid"));
            // sequencer=
            // veRecordedProgram();
            //sequencer.start();
            //sequencer.close();

            System.out.println("*************");
        }catch(Exception e)
        {
            System.out.println("RF EXCEPTION");
            e.printStackTrace();
        }
        saveMidiFile(new File( "test.mid"));
        rf.play();
    }
    public static void main(String args[]) {
        final MidiSynth midiSynth = new MidiSynth();
        midiSynth.open();
       // midiSynth.test();
        midiSynth.rec();

    }
}
