package business;

import access.DatabaseConstants;
import access.Database;

import javax.sound.midi.*;
import javax.sound.sampled.*;
import java.util.Vector;
import java.util.Hashtable;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.BufferedInputStream;

public class NewRecorder implements DatabaseConstants, ControlContext
{
    private final int PROGRAM = 192;
    private final int NOTEON = 144;
    private final int NOTEOFF = 128;
    private Sequencer sequencer;
    private Sequence sequence;
    private Synthesizer synthesizer;
    private Instrument objInstruments[];
    private ChannelData channels[];
    private ChannelData cc;    // current channel
    private final int DEFAULT_NOTE = 90;
    private RecordMIDI recordMIDI;
    private Hashtable htInstruments = null;

    //recording info
    String sProgramName=null;
    Track objTrack;
    long lStartTime;

    public NewRecorder(String sProgramName)
    {
        this.sProgramName = sProgramName;
        this.objTrack=null;
        this.lStartTime=0;
        open();
        mapInstrumentNames();
    }

    public void open() {
         try {
             if (synthesizer == null) {
                 if ((synthesizer = MidiSystem.getSynthesizer()) == null) {
                     System.out.println("getSynthesizer() failed!");
                     return;
                 }
             }
             synthesizer.open();
             sequencer = MidiSystem.getSequencer();
             sequence = new Sequence(Sequence.PPQ, 10);
         } catch (Exception ex) { ex.printStackTrace(); return; }

         Soundbank sb = synthesizer.getDefaultSoundbank();
     if (sb != null) {
             objInstruments = synthesizer.getDefaultSoundbank().getInstruments();
             synthesizer.loadInstrument(objInstruments[0]);
         }
         MidiChannel midiChannels[] = synthesizer.getChannels();
         channels = new NewRecorder.ChannelData[midiChannels.length];
         for (int i = 0; i < channels.length; i++) {
             channels[i] = new NewRecorder.ChannelData(midiChannels[i], i);
         }
         cc = channels[0];

     }


     public void close() {
         if (synthesizer != null) {
             synthesizer.close();
         }
         if (sequencer != null) {
             sequencer.close();
         }
         sequencer = null;
         synthesizer = null;
         objInstruments = null;
         channels = null;
         if (recordMIDI != null) {
             recordMIDI = null;
         }
     }




    private void mapInstrumentNames()
     {

        for(int iLoop=0; iLoop < objInstruments.length; iLoop++)
        {
             htInstruments.put(objInstruments[iLoop].getName().trim(),new Integer(iLoop));
             //System.out.println("MAP------INSTR-"+htInstruments.get(objInstruments[iLoop].getName())+"--VAL---"+htInstruments.get(objInstruments[iLoop].getName()));

        }

     }

     /**
      * Stores MidiChannel information.
      */

     class ChannelData {

         MidiChannel channel;
         boolean solo, mono, mute, sustain;
         int velocity, pressure, bend, reverb;
         int num;

         public ChannelData(MidiChannel channel, int num) {
             this.channel = channel;
             this.num = num;
             velocity = pressure = bend = reverb = 64;
         }
       } // End class ChannelData



    public void createShortEvent(int type, int num) {
            ShortMessage message = new ShortMessage();
            try {
                long millis = System.currentTimeMillis() - lStartTime;
                long tick = millis * sequence.getResolution() / 500;
                message.setMessage(type+cc.num, num, cc.velocity);
                MidiEvent event = new MidiEvent(message, tick);
                objTrack.add(event);

                try
                {
                    Thread.sleep(500);
                }
                catch(Exception e)
                {
                }

            } catch (Exception ex) { ex.printStackTrace(); }
     }



     /**
      * A frame that allows for midi capture & saving the captured data.
      */
     class RecordMIDI {
         Vector tracks = new Vector();
         public RecordMIDI() {
             try {
                 sequence = new Sequence(Sequence.PPQ, 10);
             } catch (Exception ex) { ex.printStackTrace(); }
           }

         public void setStartTime()
         {
             objTrack = sequence.createTrack();
             lStartTime = System.currentTimeMillis();
         }


         public void recordEvent(Vector vSoundProperties, Integer objData)
         {
             String sSoundVariable = (String)vSoundProperties.get(0);
             int iPitch = ((Integer)vSoundProperties.get(1)).intValue();
             String sTime = (String)vSoundProperties.get(2);

             int iData=DEFAULT_NOTE;

             if(objData != null)
             {
                 iData=((Integer)objData).intValue();
             }

             int iInstrument=0;
             try{
                  iInstrument= ((Integer)htInstruments.get(sSoundVariable)).intValue();
             }catch(Exception e)
             {
                System.out.println("INSTRUMENT NOT FOUND");
             }

             createShortEvent(PROGRAM, iInstrument);//instr
             createShortEvent(NOTEON, iData);       //note
             createShortEvent(NOTEOFF, iData);      //note
         }

           class TrackData extends Object {
             Integer chanNum; String name; Track track;
             public TrackData(int chanNum, String name, Track track) {
                 this.chanNum = new Integer(chanNum);
                 this.name = name;
                 this.track = track;
             }
         } // End class TrackData
     } // End class RecordFrame

     public void playRecordedSound(Sequencer sequencer)
     {
        sequencer.start();
     }
     public boolean saveMidiFile(File file) {
             try {
                // System.out.println("STARTED SAVING...................");
                 int[] fileTypes = MidiSystem.getMidiFileTypes(sequence);
                 if (fileTypes.length == 0) {
                     System.out.println("Can't save sequence");
                 } else {
                     if (MidiSystem.write(sequence, fileTypes[0], file) == -1)
                     {
                         throw new IOException("Problems writing to file");
                     }
                 }

                 return true;
                 //System.out.println("END OF SAVE PROCESS");
             } catch (SecurityException ex) {
                 System.out.println("FAILED TO SAVE");
                 return false;

             } catch (Exception ex) {
                 ex.printStackTrace();
                 return false;
             }
         }

    public boolean storeRecordedProgram()
    {
        String sFileName = RECORDED_PROGRAM + sProgramName + RECORDED_PROGRAM_FILE_EXTENSION;
        return saveMidiFile(new File(sFileName));

    }

    public Sequencer retrieveRecordedProgram()
    {
         File fProgram = new File(RECORDED_PROGRAM + sProgramName + RECORDED_PROGRAM_FILE_EXTENSION);
         //String sCurrentName =  fProgram.getName();
         Object currentSound = null;

            try {
                currentSound = AudioSystem.getAudioInputStream(fProgram);
            } catch(Exception e1) {
                // load midi as inputstreams 
                    try {
                        FileInputStream is = new FileInputStream(fProgram);
                        currentSound = new BufferedInputStream(is, 1024);
                    } catch (Exception e3) {
                            //e3.printStackTrace();
                            currentSound = null;
                            return null;
                    }
              }
        if (currentSound instanceof Sequence || currentSound instanceof BufferedInputStream) {
            try {
                sequencer.open();
                if (currentSound instanceof Sequence) {
                    sequencer.setSequence((Sequence) currentSound);
                } else {
                    sequencer.setSequence((BufferedInputStream) currentSound);
                }
            } catch (InvalidMidiDataException imde) {
                    System.out.println("Unsupported audio file.");
                    currentSound = null;
                    return null;
            } catch (Exception ex) {
                    //ex.printStackTrace();
                    currentSound = null;
                    return null;
	        }
        }
        return sequencer;
}


}

