package business;


import access.DatabaseConstants;
import javax.sound.midi.*;
import javax.sound.sampled.*;
import java.util.Vector;
import java.util.Hashtable;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.BufferedInputStream;

public class Recorder implements DatabaseConstants, ControlContext, MIDIConstants, MetaEventListener
{
    /* private final int PROGRAM = 192;
    private final int NOTEON = 144;
    private final int NOTEOFF = 128;
    private final int NOTE_ON = 44;
    private final int NOTE_OFF = 28;
    private final int MIDI_INSTRUMENTS_LENGTH=128;
    private final int SOUND_DELAY=500;*/


    private Sequencer sequencer;
    private Sequence sequence;
    private Synthesizer synthesizer;
    private Instrument objInstruments[];
    private ChannelData channels[];
    private ChannelData cc;    // current channel

    private Hashtable htInstruments = null;

    private final int CHANNEL_NUMBER=0;

    //recording info
    String sProgramName=null;
    Track objTrack;
    long lStartTime;

    public Recorder(String sProgramName)
    {
        htInstruments=new Hashtable();
        this.sProgramName = sProgramName;
        this.objTrack=null;
        this.lStartTime=0;
        open();
        mapInstrumentNames();
        cc.channel.controlChange(REVERB, cc.reverb);
        cc.channel.setChannelPressure(cc.pressure);
        cc.channel.setPitchBend(cc.bend);

    }

    public void open() {
        try {
            if (synthesizer == null) {
                if ((synthesizer = MidiSystem.getSynthesizer()) == null) {
                    System.out.println("getSynthesizer() failed!");
                    return;
                }
            }
            synthesizer.open();
            sequencer = MidiSystem.getSequencer();
sequencer.addMetaEventListener(this);
            sequence = new Sequence(Sequence.PPQ, 10);
        } catch (Exception ex) { ex.printStackTrace(); return; }

        Soundbank sb = synthesizer.getDefaultSoundbank();
        if (sb != null) {
            objInstruments = synthesizer.getDefaultSoundbank().getInstruments();
            synthesizer.loadInstrument(objInstruments[0]);
        }
        MidiChannel midiChannels[] = synthesizer.getChannels();
        channels = new Recorder.ChannelData[midiChannels.length];
        for (int i = 0; i < channels.length; i++) {
            channels[i] = new Recorder.ChannelData(midiChannels[i], i);
        }
        cc = channels[CHANNEL_NUMBER];

    }


    public void close() {
        if (synthesizer != null) {
            synthesizer.close();
        }
        if (sequencer != null) {
            sequencer.close();
        }
        sequencer = null;
        synthesizer = null;
        objInstruments = null;
        channels = null;

    }




    private void mapInstrumentNames()
    {

        for(int iLoop=0; iLoop < MIDI_INSTRUMENTS_LENGTH; iLoop++)
        {
            htInstruments.put(objInstruments[iLoop].getName().trim(),new Integer(iLoop));
            //System.out.println("MAP------INSTR-"+htInstruments.get(objInstruments[iLoop].getName())+"--VAL---"+htInstruments.get(objInstruments[iLoop].getName()));

        }

    }

    /**
     * Stores MidiChannel information.
     */

    class ChannelData {

        MidiChannel channel;
        boolean solo, mono, mute, sustain;
        int velocity, pressure, bend, reverb;
        int num;

        public ChannelData(MidiChannel channel, int num) {
            this.channel = channel;
            this.num = num;
            velocity = pressure = reverb = 64;
            bend=8192;
        }
    } // End class ChannelData



    private void createShortEvent(int type, int num) {
        ShortMessage message = new ShortMessage();
        try {
            long millis = System.currentTimeMillis() - lStartTime;
            long tick = millis * sequence.getResolution() / 500;
            message.setMessage(type+cc.num, num, cc.velocity);
            MidiEvent event = new MidiEvent(message, tick);
            objTrack.add(event);

            try
            {
                Thread.sleep(SOUND_DELAY);
            }
            catch(Exception e)
            {
            }

        } catch (Exception ex) { System.out.println("INVALID MIDI DATA"); }
    }

    public void setStartTime()
    {
        objTrack = sequence.createTrack();
        lStartTime = System.currentTimeMillis();
    }

    public void recordEvent(Vector vSoundProperties, Integer objData)
    {
        if(vSoundProperties != null)
        {
            String sSoundVariable = (String)vSoundProperties.get(0);
            int iNote = ((Integer)vSoundProperties.get(1)).intValue();
            String sTime = (String)vSoundProperties.get(2);

            //int iData=NOTE_ON;
            int iData=iNote;

            if(objData != null)
            {
                iData=((Integer)objData).intValue();
               //iData=(iData%MAX_NOTES == 0)?NOTE_ON:iData;
                    iData=((Integer)objData).intValue();
               if(iData < iNote)
                iData+=iNote;
               while(iData >= MAX_NOTES)
               {
                   iData = (iData % MAX_NOTES) + iNote;
               }

            }

            int iInstrument=0;
            try{
                iInstrument= ((Integer)htInstruments.get(sSoundVariable)).intValue();
            }catch(Exception e)
            {
                System.out.println("INSTRUMENT NOT FOUND");
            }

            System.out.println("INST= "+iInstrument+"   NOTE=on "+iData);
            createShortEvent(PROGRAM, iInstrument);//instr
            createShortEvent(NOTEON, iData);       //note
            try
            {
                Thread.sleep(SOUND_DELAY);
            }
            catch(Exception e)
            {
            }
            createShortEvent(NOTEOFF, iData);      //note
        }
    }


     public void meta(MetaMessage message) {
            if (message.getType() == 47) {// 47 is end of track
                System.out.println("Enabling Playing status");

               //ListenerManager.enablePlayingStatus();
            }
        }
    public void playRecordedSound()
    {

        System.out.println("Sequencer in Play : " + sequencer);
        sequencer.start();
    }

    public void stopRecordedSound()
    {
        System.out.println("Sequencer in Stop : " + sequencer);
        sequencer.stop();
    }
    public boolean saveMidiFile(File file) {
        try {
            // System.out.println("STARTED SAVING...................");
            int[] fileTypes = MidiSystem.getMidiFileTypes(sequence);
            if (fileTypes.length == 0) {
                System.out.println("Can't save sequence");
            } else {
                if (MidiSystem.write(sequence, fileTypes[0], file) == -1)
                {
                    throw new IOException("Problems writing to file");
                }
            }

            return true;
            //System.out.println("END OF SAVE PROCESS");
        } catch (SecurityException ex) {
            System.out.println("FAILED TO SAVE");
            return false;

        }
        catch (Exception ex) {
            System.out.println("&^&^&^&^&^&^&^");
            ex.printStackTrace();
            return false;
        }
    }

    public boolean storeRecordedProgram()
    {
        String sFileName = RECORDED_PROGRAM + sProgramName + RECORDED_PROGRAM_FILE_EXTENSION;
        return saveMidiFile(new File(sFileName));

    }

    public Sequencer retrieveRecordedProgram()
    {
        File fProgram = new File(RECORDED_PROGRAM + sProgramName + RECORDED_PROGRAM_FILE_EXTENSION);
        //String sCurrentName =  fProgram.getName();
        Object currentSound = null;

        try {
            currentSound = AudioSystem.getAudioInputStream(fProgram);
        } catch(Exception e1) {
            // load midi as inputstreams
            try {
                FileInputStream is = new FileInputStream(fProgram);
                currentSound = new BufferedInputStream(is, 1024);
            } catch (Exception e3) {
                //e3.printStackTrace();
                currentSound = null;
                return null;
            }
        }
        if (currentSound instanceof Sequence || currentSound instanceof BufferedInputStream) {
            try {
                sequencer.open();
                if (currentSound instanceof Sequence) {
                    sequencer.setSequence((Sequence) currentSound);
                } else {
                    sequencer.setSequence((BufferedInputStream) currentSound);
                }
            } catch (InvalidMidiDataException imde) {
                System.out.println("Unsupported audio file.");
                currentSound = null;
                return null;
            } catch (Exception ex) {
                //ex.printStackTrace();
                currentSound = null;
                return null;
            }
        }
        this.sequencer=sequencer;
        return sequencer;
    }

    public void playNote(int iData)
    {
        cc.channel.noteOn(iData, cc.velocity);
        cc.channel.noteOff(iData, cc.velocity);
        try
        {
            Thread.sleep(SOUND_DELAY);
        }catch(Exception e)
        {

        }
    }

    public void mute(boolean bStatus)
    {
        cc.channel.setMute(cc.mute=bStatus);
    }

}
/*
  createShortEvent(PROGRAM, 73);//instr
            createShortEvent(NOTEON, iData);       //note
            try
            {
                Thread.sleep(SOUND_DELAY);
            }catch(Exception e)
            {
            }
  createShortEvent(NOTEOFF, iData);      //note

*/
